@@ -1,22 +1,32 @@
 # -*- coding: utf-8 -*-
 # This file is part of Xpra.
-# Copyright (C) 2010-2018 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2010-2018 Antoine Martin <antoine@xpra.org>
 # Copyright (C) 2008 Nathaniel Smith <njs@pobox.com>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
 import os
 
-from xpra.log import Logger
-log = Logger("server")
-metalog = Logger("metadata")
-
-from xpra.util import WORKSPACE_UNSET
+from xpra.util import WORKSPACE_UNSET, get_util_logger
 
 SKIP_METADATA = os.environ.get("XPRA_SKIP_METADATA", "").split(",")
 
 
-def make_window_metadata(window, propname, get_transient_for=None, get_window_id=None, skip_defaults=False):
+def make_window_metadata(window, propname, get_transient_for=None, get_window_id=None, skip_defaults=False) -> dict:
+    try:
+        return do_make_window_metadata(window, propname, get_transient_for, get_window_id, skip_defaults)
+    except (ValueError, TypeError) as e:
+        log = get_util_logger()
+        log("make_window_metadata%s",
+            (window, propname, get_transient_for, get_window_id, skip_defaults), exc_info=True)
+        log.error("Error: failed to make window metadata")
+        log.error(" for attribute '%s' of window %s", propname, window)
+        log.error(" with value '%s':", getattr(window, propname, None))
+        log.error(" %s", e)
+        return {}
+
+
+def do_make_window_metadata(window, propname, get_transient_for=None, get_window_id=None, skip_defaults=False) -> dict:
     if propname in SKIP_METADATA:
         return {}
     #note: some of the properties handled here aren't exported to the clients,
@@ -30,21 +40,27 @@ def make_window_metadata(window, propname, get_transient_for=None, get_window_id
                 return {}
             return {propname: ""}
         return {propname: v.encode("utf-8")}
-    elif propname in ("pid", "workspace", "bypass-compositor", "depth", "opacity"):
+    if propname in ("pid", "workspace", "bypass-compositor", "depth", "opacity", "quality", "speed"):
         v = raw()
         assert v is not None, "%s is None!" % propname
-        if ((v<0) or
-            (v==WORKSPACE_UNSET and propname=="workspace") or
-            (v==0 and propname=="bypass-compositor")) and skip_defaults:
-            #unset, so don't bother sending anything
+        default_value = {
+            "pid"               : 0,
+            "workspace"         : WORKSPACE_UNSET,
+            "bypass-compositor" : 0,
+            "depth"             : 24,
+            "opacity"           : 100,
+            }.get(propname, -1)
+        if (v<0 or v==default_value) and skip_defaults:
+            #unset or default value,
+            #so don't bother sending anything:
             return {}
         return {propname : v}
-    elif propname == "size-hints":
+    if propname == "size-hints":
         #just to confuse things, this is renamed
         #and we have to filter out ratios as floats (already exported as pairs anyway)
-        v = dict((k,v) for k,v in raw().items() if k not in("max_aspect", "min_aspect"))
+        v = dict(raw())
         return {"size-constraints": v}
-    elif propname == "strut":
+    if propname == "strut":
         strut = raw()
         if not strut:
             strut = {}
@@ -53,12 +69,12 @@ def make_window_metadata(window, propname, get_transient_for=None, get_window_id
         if not strut and skip_defaults:
             return {}
         return {"strut": strut}
-    elif propname == "class-instance":
+    if propname == "class-instance":
         c_i = raw()
         if c_i is None:
             return {}
         return {"class-instance": [x.encode("utf-8") for x in c_i]}
-    elif propname == "client-machine":
+    if propname == "client-machine":
         client_machine = raw()
         if client_machine is None:
             import socket
@@ -66,26 +82,31 @@ def make_window_metadata(window, propname, get_transient_for=None, get_window_id
             if client_machine is None:
                 return {}
         return {"client-machine": client_machine.encode("utf-8")}
-    elif propname == "transient-for":
+    if propname == "transient-for":
         wid = None
         if get_transient_for:
             wid = get_transient_for(window)
         if wid:
             return {"transient-for" : wid}
         return {}
-    elif propname in ("window-type", "shape", "menu"):
+    if propname in ("window-type", "shape", "children"):
         v = raw()
         if not v and skip_defaults:
             return {}
         #always send unchanged:
         return {propname : raw()}
-    elif propname in ("decorations", ):
+    if propname=="decorations":
         #-1 means unset, don't send it
         v = raw()
         if v<0:
             return {}
         return {propname : v}
-    elif propname in ("iconic", "fullscreen", "maximized", "above", "below", "shaded", "sticky", "skip-taskbar", "skip-pager", "modal", "focused"):
+    if propname in ("iconic", "fullscreen", "maximized",
+                      "above", "below",
+                      "shaded", "sticky",
+                      "skip-taskbar", "skip-pager",
+                      "modal", "focused",
+                      ):
         v = raw()
         if v is False and skip_defaults:
             #we can skip those when the window is first created,
@@ -93,20 +114,20 @@ def make_window_metadata(window, propname, get_transient_for=None, get_window_id
             return {}
         #always send these when requested
         return {propname : bool(raw())}
-    elif propname in ("has-alpha", "override-redirect", "tray", "shadow", "set-initial-position"):
+    if propname in ("has-alpha", "override-redirect", "tray", "shadow", "set-initial-position"):
         v = raw()
         if v is False and skip_defaults:
             #save space: all these properties are assumed false if unspecified
             return {}
         return {propname : v}
-    elif propname in ("role", "fullscreen-monitors"):
+    if propname in ("role", "fullscreen-monitors"):
         v = raw()
         if v is None or v=="":
             return {}
         return {propname : v}
-    elif propname == "xid":
+    if propname == "xid":
         return {"xid" : hex(raw() or 0)}
-    elif propname == "group-leader":
+    if propname == "group-leader":
         gl = raw()
         if not gl or not get_window_id:
             return  {}
@@ -122,11 +143,11 @@ def make_window_metadata(window, propname, get_transient_for=None, get_window_id
     #the properties below are not actually exported to the client (yet?)
     #it was just easier to handle them here
     #(convert to a type that can be encoded for xpra info):
-    elif propname in ("state", "protocols"):
+    if propname in ("state", "protocols"):
         return {"state" : tuple(raw() or [])}
-    elif propname == "allowed-actions":
+    if propname == "allowed-actions":
         return {"allowed-actions" : tuple(raw())}
-    elif propname == "frame":
+    if propname == "frame":
         frame = raw()
         if not frame:
             return {}
