@@ -1,52 +1,27 @@
 # This file is part of Xpra.
 # Copyright (C) 2008, 2009 Nathaniel Smith <njs@pobox.com>
-# Copyright (C) 2011-2019 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2011-2020 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
-import os
+from gi.repository import GObject
 
 from xpra.util import WORKSPACE_UNSET, WORKSPACE_ALL
 from xpra.x11.models.core import CoreX11WindowModel, xswallow
 from xpra.x11.bindings.window_bindings import X11WindowBindings, constants      #@UnresolvedImport
+from xpra.server.window.content_guesser import guess_content_type, get_content_type_properties
 from xpra.x11.gtk_x11.gdk_bindings import get_pywindow, get_pyatom              #@UnresolvedImport
-from xpra.x11.gtk_x11.prop import prop_set, prop_get
-from xpra.gtk_common.gtk_util import atom_intern, PARAM_READABLE, PARAM_READWRITE
-from xpra.gtk_common.gobject_compat import import_gobject
+from xpra.x11.gtk_x11.prop import prop_set, prop_get, prop_del
 from xpra.log import Logger
 
-gobject = import_gobject()
-
 log = Logger("x11", "window")
 workspacelog = Logger("x11", "window", "workspace")
 metalog = Logger("x11", "window", "metadata")
 geomlog = Logger("x11", "window", "geometry")
-menulog = Logger("x11", "window", "menu")
 
 
 dbus_helper = None
 query_actions = None
-MENU_FORWARDING = os.environ.get("XPRA_MENU_FORWARDING", "1")=="1"
-if MENU_FORWARDING:
-    try:
-        from xpra import dbus
-        assert dbus
-    except ImportError as e:
-        menulog("this build does not include the dbus module, no menu forwarding")
-        MENU_FORWARDING = False
-        del e
-    else:
-        try:
-            from xpra.dbus.helper import DBusHelper
-            dbus_helper = DBusHelper()
-            from xpra.dbus.gtk_menuactions import query_actions, query_menu, ACTIONS, MENUS
-        except Exception as e:
-            menulog("menu actions", exc_info=True)
-            menulog.warn("Warning: menu forwarding is disabled")
-            menulog.warn(" cannot load dbus helper:",)
-            for msg in str(e).split(": "):
-                menulog.warn(" %s", msg)
-            del e
 
 
 X11Window = X11WindowBindings()
@@ -83,136 +58,132 @@ class BaseWindowModel(CoreX11WindowModel):
     __common_properties__ = CoreX11WindowModel.__common_properties__.copy()
     __common_properties__.update({
         #from WM_TRANSIENT_FOR
-        "transient-for": (gobject.TYPE_PYOBJECT,
+        "transient-for": (GObject.TYPE_PYOBJECT,
                           "Transient for (or None)", "",
-                          PARAM_READABLE),
+                          GObject.ParamFlags.READABLE),
         #from _NET_WM_WINDOW_OPACITY
-        "opacity": (gobject.TYPE_INT64,
+        "opacity": (GObject.TYPE_INT64,
                 "Opacity", "",
                 -1, 0xffffffff, -1,
-                PARAM_READABLE),
+                GObject.ParamFlags.READABLE),
         #from WM_HINTS.window_group
-        "group-leader": (gobject.TYPE_PYOBJECT,
+        "group-leader": (GObject.TYPE_PYOBJECT,
                          "Window group leader as a pair: (xid, gdk window)", "",
-                         PARAM_READABLE),
+                         GObject.ParamFlags.READABLE),
         #from WM_HINTS.urgency or _NET_WM_STATE
-        "attention-requested": (gobject.TYPE_BOOLEAN,
+        "attention-requested": (GObject.TYPE_BOOLEAN,
                                 "Urgency hint from client, or us", "",
                                 False,
-                                PARAM_READWRITE),
+                                GObject.ParamFlags.READWRITE),
         #from WM_HINTS.input or WM_TAKE_FOCUS
-        "can-focus": (gobject.TYPE_BOOLEAN,
+        "can-focus": (GObject.TYPE_BOOLEAN,
                       "Does this window ever accept keyboard input?", "",
                       True,
-                      PARAM_READWRITE),
+                      GObject.ParamFlags.READWRITE),
         #from _NET_WM_BYPASS_COMPOSITOR
-        "bypass-compositor": (gobject.TYPE_INT,
+        "bypass-compositor": (GObject.TYPE_INT,
                        "hint that the window would benefit from running uncomposited ", "",
                        0, 2, 0,
-                       PARAM_READABLE),
+                       GObject.ParamFlags.READABLE),
         #from _NET_WM_FULLSCREEN_MONITORS
-        "fullscreen-monitors": (gobject.TYPE_PYOBJECT,
+        "fullscreen-monitors": (GObject.TYPE_PYOBJECT,
                          "List of 4 monitor indices indicating the top, bottom, left, and right edges"+
                          " of the window when the fullscreen state is enabled", "",
-                         PARAM_READABLE),
+                         GObject.ParamFlags.READABLE),
         #from _NET_WM_STRUT_PARTIAL or _NET_WM_STRUT
-        "strut": (gobject.TYPE_PYOBJECT,
+        "strut": (GObject.TYPE_PYOBJECT,
                   "Struts requested by window, or None", "",
-                  PARAM_READABLE),
-        #from _GTK_*MENU*
-        "menu": (gobject.TYPE_PYOBJECT,
-                  "Application menu, or None", "",
-                  PARAM_READABLE),
+                  GObject.ParamFlags.READABLE),
         #for our own use:
-        "content-type": (gobject.TYPE_PYOBJECT,
+        "content-type": (GObject.TYPE_PYOBJECT,
                   "What type of content is shown in this window", "",
-                  PARAM_READABLE),
+                  GObject.ParamFlags.READABLE),
         #from _XPRA_QUALITY
-        "quality": (gobject.TYPE_INT,
+        "quality": (GObject.TYPE_INT,
                 "Quality", "",
                 -1, 100, -1,
-                PARAM_READABLE),
+                GObject.ParamFlags.READABLE),
         #from _XPRA_SPEED
-        "speed": (gobject.TYPE_INT,
+        "speed": (GObject.TYPE_INT,
                 "Speed", "",
                 -1, 100, -1,
-                PARAM_READABLE),
+                GObject.ParamFlags.READABLE),
         #from _XPRA_ENCODING
-        "encoding": (gobject.TYPE_PYOBJECT,
+        "encoding": (GObject.TYPE_PYOBJECT,
                 "Encoding", "",
-                PARAM_READABLE),
+                GObject.ParamFlags.READABLE),
         #from _NET_WM_DESKTOP
-        "workspace": (gobject.TYPE_UINT,
+        "workspace": (GObject.TYPE_UINT,
                 "The workspace this window is on", "",
                 0, 2**32-1, WORKSPACE_UNSET,
-                PARAM_READWRITE),
+                GObject.ParamFlags.READWRITE),
         #set initially only by the window model class
         #(derived from XGetWindowAttributes.override_redirect)
-        "override-redirect": (gobject.TYPE_BOOLEAN,
+        "override-redirect": (GObject.TYPE_BOOLEAN,
                        "Is the window of type override-redirect", "",
                        False,
-                       PARAM_READABLE),
+                       GObject.ParamFlags.READABLE),
         #from _NET_WM_WINDOW_TYPE
-        "window-type": (gobject.TYPE_PYOBJECT,
+        "window-type": (GObject.TYPE_PYOBJECT,
                         "Window type",
                         "NB, most preferred comes first, then fallbacks",
-                        PARAM_READABLE),
+                        GObject.ParamFlags.READABLE),
         #this value is synced to "_NET_WM_STATE":
-        "state": (gobject.TYPE_PYOBJECT,
+        "state": (GObject.TYPE_PYOBJECT,
                   "State, as per _NET_WM_STATE", "",
-                  PARAM_READABLE),
+                  GObject.ParamFlags.READABLE),
         #all the attributes below are virtual attributes from WM_STATE:
-        "modal": (gobject.TYPE_PYOBJECT,
+        "modal": (GObject.TYPE_PYOBJECT,
                           "Modal", "",
-                          PARAM_READWRITE),
-        "fullscreen": (gobject.TYPE_BOOLEAN,
+                          GObject.ParamFlags.READWRITE),
+        "fullscreen": (GObject.TYPE_BOOLEAN,
                        "Fullscreen-ness of window", "",
                        False,
-                       PARAM_READWRITE),
-        "focused": (gobject.TYPE_BOOLEAN,
+                       GObject.ParamFlags.READWRITE),
+        "focused": (GObject.TYPE_BOOLEAN,
                        "Is the window focused", "",
                        False,
-                       PARAM_READWRITE),
-        "maximized": (gobject.TYPE_BOOLEAN,
+                       GObject.ParamFlags.READWRITE),
+        "maximized": (GObject.TYPE_BOOLEAN,
                        "Is the window maximized", "",
                        False,
-                       PARAM_READWRITE),
-        "above": (gobject.TYPE_BOOLEAN,
+                       GObject.ParamFlags.READWRITE),
+        "above": (GObject.TYPE_BOOLEAN,
                        "Is the window on top of most windows", "",
                        False,
-                       PARAM_READWRITE),
-        "below": (gobject.TYPE_BOOLEAN,
+                       GObject.ParamFlags.READWRITE),
+        "below": (GObject.TYPE_BOOLEAN,
                        "Is the window below most windows", "",
                        False,
-                       PARAM_READWRITE),
-        "shaded": (gobject.TYPE_BOOLEAN,
+                       GObject.ParamFlags.READWRITE),
+        "shaded": (GObject.TYPE_BOOLEAN,
                        "Is the window shaded", "",
                        False,
-                       PARAM_READWRITE),
-        "skip-taskbar": (gobject.TYPE_BOOLEAN,
+                       GObject.ParamFlags.READWRITE),
+        "skip-taskbar": (GObject.TYPE_BOOLEAN,
                        "Should the window be included on a taskbar", "",
                        False,
-                       PARAM_READWRITE),
-        "skip-pager": (gobject.TYPE_BOOLEAN,
+                       GObject.ParamFlags.READWRITE),
+        "skip-pager": (GObject.TYPE_BOOLEAN,
                        "Should the window be included on a pager", "",
                        False,
-                       PARAM_READWRITE),
-        "sticky": (gobject.TYPE_BOOLEAN,
+                       GObject.ParamFlags.READWRITE),
+        "sticky": (GObject.TYPE_BOOLEAN,
                        "Is the window's position fixed on the screen", "",
                        False,
-                       PARAM_READWRITE),
+                       GObject.ParamFlags.READWRITE),
         })
     _property_names = CoreX11WindowModel._property_names + [
                       "transient-for", "fullscreen-monitors", "bypass-compositor",
                       "group-leader", "window-type", "workspace", "strut", "opacity",
-                      "menu", "content-type",
+                      "content-type",
                       #virtual attributes:
                       "fullscreen", "focused", "maximized", "above", "below", "shaded",
                       "skip-taskbar", "skip-pager", "sticky",
                       ]
     _dynamic_property_names = CoreX11WindowModel._dynamic_property_names + [
                               "attention-requested", "content-type",
-                              "menu", "workspace", "opacity",
+                              "workspace", "opacity",
                               "fullscreen", "focused", "maximized", "above", "below", "shaded",
                               "skip-taskbar", "skip-pager", "sticky",
                               "quality", "speed", "encoding",
@@ -229,18 +200,10 @@ class BaseWindowModel(CoreX11WindowModel):
                               "_NET_WM_STRUT_PARTIAL",
                               "_NET_WM_WINDOW_OPACITY",
                               "WM_HINTS",
-                              "_GTK_APP_MENU_OBJECT_PATH",
                               "_XPRA_CONTENT_TYPE",
                               "_XPRA_QUALITY",
                               "_XPRA_SPEED",
                               "_XPRA_ENCODING",
-                              #redundant as they use the same function as _GTK_APP_MENU_OBJECT_PATH:
-                              #(but the code will make sure we only call it once during setup)
-                              "_GTK_APPLICATION_ID",
-                              "_GTK_UNIQUE_BUS_NAME",
-                              "_GTK_APPLICATION_OBJECT_PATH",
-                              "_GTK_APP_MENU_OBJECT_PATH",
-                              "_GTK_WINDOW_OBJECT_PATH",
                               ]
     _DEFAULT_NET_WM_ALLOWED_ACTIONS = ["_NET_WM_ACTION_%s" % x for x in (
         "CLOSE", "MOVE", "RESIZE", "FULLSCREEN",
@@ -250,11 +213,15 @@ class BaseWindowModel(CoreX11WindowModel):
     _MODELTYPE = "Base"
 
     def __init__(self, client_window):
-        super(BaseWindowModel, self).__init__(client_window)
+        super().__init__(client_window)
         self.last_unmap_serial = 0
         self._input_field = True            # The WM_HINTS input field
+        #watch for changes to properties that are used to derive the content-type:
+        for x in get_content_type_properties():
+            if x in self.get_dynamic_property_names():
+                self.connect("notify::%s" % x, self._content_type_related_property_change)
 
-    def serial_after_last_unmap(self, serial):
+    def serial_after_last_unmap(self, serial) -> bool:
         #"The serial member is set from the serial number reported in the protocol
         # but expanded from the 16-bit least-significant bits to a full 32-bit value"
         if serial>self.last_unmap_serial:
@@ -268,9 +235,9 @@ class BaseWindowModel(CoreX11WindowModel):
     def _read_initial_X11_properties(self):
         metalog("%s.read_initial_X11_properties()", self._MODELTYPE)
         self._updateprop("state", frozenset(self._read_wm_state()))
-        super(BaseWindowModel, self)._read_initial_X11_properties()
+        super()._read_initial_X11_properties()
 
-    def _guess_window_type(self):
+    def _guess_window_type(self) -> str:
         #query the X11 property directly,
         #in case the python property isn't set yet
         if not self.is_OR():
@@ -288,7 +255,7 @@ class BaseWindowModel(CoreX11WindowModel):
     # Actions
     ################################
 
-    def move_to_workspace(self, workspace):
+    def move_to_workspace(self, workspace : int):
         #we send a message to ourselves, we could also just update the property
         current = self.get_property("workspace")
         if current==workspace:
@@ -298,7 +265,7 @@ class BaseWindowModel(CoreX11WindowModel):
         with xswallow:
             if workspace==WORKSPACE_UNSET:
                 workspacelog("removing _NET_WM_DESKTOP property from window %#x", self.xid)
-                X11Window.XDeleteProperty(self.xid, "_NET_WM_DESKTOP")
+                prop_del(self.client_window, "_NET_WM_DESKTOP")
             else:
                 workspacelog("setting _NET_WM_DESKTOP=%s on window %#x", workspacestr(workspace), self.xid)
                 prop_set(self.client_window, "_NET_WM_DESKTOP", "u32", workspace)
@@ -351,7 +318,8 @@ class BaseWindowModel(CoreX11WindowModel):
         if not window_types:
             window_type = self._guess_window_type()
             metalog("guessed window type=%s", window_type)
-            window_types = [atom_intern(window_type)]
+            #atom = Gdk.Atom.intern(window_type, False)
+            window_types = [window_type]
         #normalize them (hide _NET_WM_WINDOW_TYPE prefix):
         window_types = [str(wt).replace("_NET_WM_WINDOW_TYPE_", "").replace("_NET_WM_TYPE_", "") for wt in window_types]
         self._updateprop("window-type", window_types)
@@ -399,7 +367,7 @@ class BaseWindowModel(CoreX11WindowModel):
             xid = wm_hints.get("window_group")
             try:
                 group_leader = xid, get_pywindow(xid)
-            except:
+            except Exception:
                 group_leader = xid, None
         self._updateprop("group-leader", group_leader)
         self._updateprop("attention-requested", wm_hints.get("urgency", False))
@@ -417,112 +385,22 @@ class BaseWindowModel(CoreX11WindowModel):
         self._updateprop("can-focus", can_focus)
 
 
-    def _get_x11_menu_properties(self):
-        props = {}
-        for k,x11name in {
-            "application-id"    : "_GTK_APPLICATION_ID",            #ie: org.gnome.baobab
-            "bus-name"          : "_GTK_UNIQUE_BUS_NAME",           #ie: :1.745
-            "application-path"  : "_GTK_APPLICATION_OBJECT_PATH",   #ie: /org/gnome/baobab
-            "app-menu-path"     : "_GTK_APP_MENU_OBJECT_PATH",      #ie: /org/gnome/baobab/menus/appmenu
-            "window-path"       : "_GTK_WINDOW_OBJECT_PATH",        #ie: /org/gnome/baobab/window/1
-            }.items():
-            v = self.prop_get(x11name, "utf8", ignore_errors=True, raise_xerrors=False)
-            menulog("%s=%s", x11name, v)
-            if v:
-                props[k] = v
-        return props
-
-    def _handle_gtk_app_menu_change(self):
-        def nomenu(*_args):
-            self._updateprop("menu", {})
-        if not query_actions:
-            nomenu()
-        props = self._get_x11_menu_properties()
-        if len(props)<5:        #incomplete
-            nomenu()
+    def _content_type_related_property_change(self, *_args):
+        #watch for changes to properties that are used to derive the content-type:
+        content_type = self.prop_get("_XPRA_CONTENT_TYPE", "latin1", True)
+        if content_type:
+            #the _XPRA_CONTENT_TYPE property takes precedence
             return
-        menulog("gtk menu properties: %s", props)
-        app_id      = props["application-id"]
-        bus_name    = props["bus-name"]
-        app_path    = props["application-path"]
-        menu_path   = props["app-menu-path"]
-        window_path = props["window-path"]
-        def updatemenu(k, v):
-            """ Updates the 'menu' property if the value has changed.
-                Sets the 'enabled' flag if all the required properties are present.
-            """
-            menu = self._gproperties.get("menu") or {}
-            cv = menu.get(k)
-            if cv==v:
-                return
-            if v is None:
-                try:
-                    del menu[k]
-                except KeyError:
-                    pass
-            else:
-                menu[k] = v
-            enabled = all(menu.get(x) for x in (
-                "application-id",
-                "application-actions",
-                "window-actions",
-                "window-menu",
-                ))
-            menu["enabled"] = enabled
-            menulog("menu(%s)=%s", self, menu)
-            self._internal_set_property("menu", menu)
-        updatemenu("application-id", app_id)
-        def app_actions_cb(actions):
-            menulog("application-actions=%s", actions)
-            updatemenu("application-actions", actions)
-        def app_actions_err(err):
-            menulog.error("Error: failed to query %s at %s:", ACTIONS, app_path)
-            menulog.error(" %s", err)
-            updatemenu("application-actions", None)
-        query_actions(bus_name, app_path, app_actions_cb, app_actions_err)
-        def window_actions_cb(actions):
-            menulog("window-actions", actions)
-            updatemenu("window-actions", actions)
-        def window_actions_err(err):
-            menulog.error("Error: failed to query %s at %s:", ACTIONS, window_path)
-            menulog.error(" %s", err)
-            updatemenu("window-actions", None)
-        query_actions(bus_name, window_path, window_actions_cb, window_actions_err)
-        def menu_cb(menu):
-            menulog("window-menu", menu)
-            updatemenu("window-menu", menu)
-        def menu_err(err):
-            menulog.error("Error: failed to query %s at %s:", MENUS, menu_path)
-            menulog.error(" %s", err)
-            updatemenu("window-menu", None)
-        query_menu(bus_name, menu_path, menu_cb, menu_err)
-
-    def activate_menu(self, action_type, action, state, pdata):
-        #locate the service and call it:
-        from xpra.dbus.gtk_menuactions import get_actions_interface
-        props = self._get_x11_menu_properties()
-        menulog("activate_menu%s properties=%s", (action_type, action, state, pdata), props)
-        if not props:
-            raise Exception("no menu defined for %s" % self)
-        bus_name    = props.get("bus-name")
-        if not bus_name:
-            raise Exception("no bus name defined for %s" % self)
-        if action_type=="application":
-            path = "application-path"
-        elif action_type=="window":
-            path = "window-path"
-        else:
-            raise Exception("unknown action type for menu: %s" % action_type)
-        object_path = props.get(path)
-        if not object_path:
-            raise Exception("no object path property %s" % path)
-        actions_iface = get_actions_interface(bus_name, object_path)
-        actions_iface.Activate(action, state, pdata)
-
+        content_type = guess_content_type(self)
+        metalog("guess_content_type(%s)=%s", self, content_type)
+        self._updateprop("content-type", content_type)
 
     def _handle_xpra_content_type_change(self):
         content_type = self.prop_get("_XPRA_CONTENT_TYPE", "latin1", True) or ""
         metalog("content_type=%s", content_type)
+        if not content_type:
+            content_type = guess_content_type(self)
+            metalog("guess_content_type(%s)=%s", self, content_type)
         self._updateprop("content-type", content_type)
 
     def _handle_xpra_quality_change(self):
@@ -552,11 +430,6 @@ class BaseWindowModel(CoreX11WindowModel):
         "_NET_WM_STRUT_PARTIAL"         : _handle_wm_strut_change,
         "_NET_WM_WINDOW_OPACITY"        : _handle_opacity_change,
         "WM_HINTS"                      : _handle_wm_hints_change,
-        "_GTK_APPLICATION_ID"           : _handle_gtk_app_menu_change,
-        "_GTK_UNIQUE_BUS_NAME"          : _handle_gtk_app_menu_change,
-        "_GTK_APPLICATION_OBJECT_PATH"  : _handle_gtk_app_menu_change,
-        "_GTK_APP_MENU_OBJECT_PATH"     : _handle_gtk_app_menu_change,
-        "_GTK_WINDOW_OBJECT_PATH"       : _handle_gtk_app_menu_change,
         "_XPRA_CONTENT_TYPE"            : _handle_xpra_content_type_change,
         "_XPRA_QUALITY"                 : _handle_xpra_quality_change,
         "_XPRA_SPEED"                   : _handle_xpra_speed_change,
@@ -644,14 +517,14 @@ class BaseWindowModel(CoreX11WindowModel):
             #virtual property for WM_STATE:
             self.update_state(pspec.name, value)
             return
-        super(BaseWindowModel, self).do_set_property(pspec, value)
+        super().do_set_property(pspec, value)
 
     def do_get_property(self, pspec):
         #intercept state properties to route via get_wm_state()
         if pspec.name in self._state_properties:
             #virtual property for WM_STATE:
             return self.get_wm_state(pspec.name)
-        return super(BaseWindowModel, self).do_get_property(pspec)
+        return super().do_get_property(pspec)
 
 
     def update_wm_state(self, prop, b):
@@ -737,7 +610,7 @@ class BaseWindowModel(CoreX11WindowModel):
                 log.info("Unhandled _NET_WM_STATE request: '%s'", event, atom1)
                 log.info(" event%s", event)
             return True
-        elif event.message_type=="WM_CHANGE_STATE":
+        if event.message_type=="WM_CHANGE_STATE":
             iconic = event.data[0]
             log("WM_CHANGE_STATE: %s, serial=%s, last unmap serial=%#x",
                 ICONIC_STATE_STRING.get(iconic, iconic), event.serial, self.last_unmap_serial)
@@ -748,16 +621,19 @@ class BaseWindowModel(CoreX11WindowModel):
                 ):
                 self._updateprop("iconic", iconic==IconicState)
             return True
-        elif event.message_type=="_NET_WM_MOVERESIZE":
+        if event.message_type=="_NET_WM_MOVERESIZE":
             log("_NET_WM_MOVERESIZE: %s", event)
             self.emit("initiate-moveresize", event)
             return True
-        elif event.message_type=="_NET_ACTIVE_WINDOW" and event.data[0] in (0, 1):
+        if event.message_type=="_NET_ACTIVE_WINDOW":
+            #to filter based on the source indication:
+            #ACTIVE_WINDOW_SOURCE = tuple(int(x) for x in os.environ.get("XPRA_ACTIVE_WINDOW_SOURCE", "0,1").split(","))
+            #if event.data[0] in ACTIVE_WINDOW_SOURCE:
             log("_NET_ACTIVE_WINDOW: %s", event)
             self.set_active()
             self.emit("raised", event)
             return True
-        elif event.message_type=="_NET_WM_DESKTOP":
+        if event.message_type=="_NET_WM_DESKTOP":
             workspace = int(event.data[0])
             #query the workspace count on the root window
             #since we cannot access Wm from here..
@@ -774,7 +650,7 @@ class BaseWindowModel(CoreX11WindowModel):
                 workspacelog.warn("invalid _NET_WM_DESKTOP request: workspace=%s, number of desktops=%s",
                                   workspacestr(workspace), ndesktops)
             return True
-        elif event.message_type=="_NET_WM_FULLSCREEN_MONITORS":
+        if event.message_type=="_NET_WM_FULLSCREEN_MONITORS":
             log("_NET_WM_FULLSCREEN_MONITORS: %s", event)
             #TODO: we should validate the indexes instead of copying them blindly!
             #TODO: keep track of source indication so we can forward that to the client
@@ -787,8 +663,14 @@ class BaseWindowModel(CoreX11WindowModel):
             log("_NET_WM_FULLSCREEN_MONITORS: monitors=%s", monitors)
             prop_set(self.client_window, "_NET_WM_FULLSCREEN_MONITORS", ["u32"], monitors)
             return True
+        if event.message_type=="_NET_RESTACK_WINDOW":
+            source = {1 : "application", 2 : "pager"}.get(event.data[0], "default (%s)" % event.data[0])
+            sibling_window = event.data[1]
+            log.info("%s not handled yet, sent to window %#x for sibling %#x from %s",
+                     event.message_type, event.window, sibling_window, source)
+            return True
         #TODO: maybe we should process _NET_MOVERESIZE_WINDOW here?
         # it may make sense to apply it to the client_window
         # whereas the code in WindowModel assumes there is a corral window
         #not handled:
-        return CoreX11WindowModel.process_client_message_event(self, event)
+        return super().process_client_message_event(event)
